@@ -1,3 +1,8 @@
+"""Execution backends.
+
+Lead authors: Johan Mathe and Niklas Koep.
+"""
+
 import logging
 import os
 import sys
@@ -89,6 +94,7 @@ BACKEND_ATTRIBUTES = {
         "polygamma",
         "power",
         "prod",
+        "ravel_tril_indices",
         "real",
         "repeat",
         "reshape",
@@ -114,9 +120,13 @@ BACKEND_ATTRIBUTES = {
         "trace",
         "transpose",
         "tril",
+        "triu",
         "tril_indices",
         "triu_indices",
+        "tril_to_vec",
         "triu_to_vec",
+        "vec_to_diag",
+        "unique",
         "vectorize",
         "vstack",
         "where",
@@ -132,13 +142,16 @@ BACKEND_ATTRIBUTES = {
         "eigvalsh",
         "expm",
         "inv",
+        "is_single_matrix_pd",
         "logm",
         "norm",
         "qr",
+        "quadratic_assignment",
         "solve",
         "solve_sylvester",
         "sqrtm",
         "svd",
+        "matrix_rank",
     ],
     "random": [
         "choice",
@@ -225,11 +238,13 @@ class BackendImporter:
         return new_module
 
     def find_module(self, fullname, path=None):
+        """Find module."""
         if self._path != fullname:
             return None
         return self
 
     def load_module(self, fullname):
+        """Load module."""
         if fullname in sys.modules:
             return sys.modules[fullname]
 
