@@ -12,6 +12,9 @@ from torch import broadcast_tensors as broadcast_arrays
 from torch import (
     ceil,
     clip,
+    complex32,
+    complex64,
+    complex128,
     cos,
     cosh,
     cross,
@@ -27,18 +30,7 @@ from torch import (
     floor,
 )
 from torch import fmod as mod
-from torch import (
-    greater,
-    hstack,
-    imag,
-    int32,
-    int64,
-    isnan,
-    less,
-    log,
-    logical_or,
-    matmul,
-)
+from torch import greater, hstack, imag, int32, int64, isnan, less, log, logical_or
 from torch import max as amax
 from torch import mean, meshgrid
 from torch import min as amin
@@ -51,13 +43,12 @@ from torch import (
     sign,
     sin,
     sinh,
-    sort,
     stack,
     std,
     tan,
     tanh,
-    tril,
     uint8,
+    unique,
     vstack,
     zeros,
     zeros_like,
@@ -68,7 +59,14 @@ from . import autodiff  # NOQA
 from . import linalg  # NOQA
 from . import random  # NOQA
 
-DTYPES = {int32: 0, int64: 1, float32: 2, float64: 3}
+DTYPES = {
+    int32: 0,
+    int64: 1,
+    float32: 2,
+    float64: 3,
+    complex64: 4,
+    complex128: 5,
+}
 
 
 atol = pytorch_atol
@@ -109,6 +107,11 @@ def comb(n, k):
     return math.factorial(n) // math.factorial(k) // math.factorial(n - k)
 
 
+def matmul(x, y, *, out=None):
+    x, y = convert_to_wider_dtype([x, y])
+    return torch.matmul(x, y, out=out)
+
+
 def to_numpy(x):
     return x.numpy()
 
@@ -499,6 +502,14 @@ def diag_indices(*args, **kwargs):
     return tuple(map(torch.from_numpy, _np.diag_indices(*args, **kwargs)))
 
 
+def tril(mat, k=0):
+    return torch.tril(mat, diagonal=k)
+
+
+def triu(mat, k=0):
+    return torch.triu(mat, diagonal=k)
+
+
 def tril_indices(n, k=0, m=None):
     if m is None:
         m = n
@@ -765,6 +776,16 @@ def vectorize(x, pyfunc, multiple_args=False, **kwargs):
     return stack(list(map(pyfunc, x)))
 
 
+def vec_to_diag(vec):
+    return torch.diag_embed(vec, offset=0)
+
+
+def tril_to_vec(x, k=0):
+    n = x.shape[-1]
+    rows, cols = tril_indices(n, k=k)
+    return x[..., rows, cols]
+
+
 def triu_to_vec(x, k=0):
     n = x.shape[-1]
     rows, cols = triu_indices(n, k=k)
@@ -795,3 +816,17 @@ def mat_from_diag_triu_tril(diag, tri_upp, tri_low):
     mat[..., j, k] = tri_upp
     mat[..., k, j] = tri_low
     return mat
+
+
+def ravel_tril_indices(n, k=0, m=None):
+    if m is None:
+        size = (n, n)
+    else:
+        size = (n, m)
+    idxs = _np.tril_indices(n, k, m)
+    return torch.from_numpy(_np.ravel_multi_index(idxs, size))
+
+
+def sort(a, axis=-1):
+    sorted_a, _ = torch.sort(a, dim=axis)
+    return sorted_a
