@@ -1,6 +1,9 @@
 """Tensorflow based random backend."""
 
 import tensorflow as tf
+import tensorflow_probability as tfp
+
+tfd = tfp.distributions
 
 
 def choice(x, size, axis=0):
@@ -21,9 +24,8 @@ def randint(low, high=None, size=None):
         maxval = low - 1
         minval = 0
     return tf.random.uniform(
-        shape=size,
-        minval=minval,
-        maxval=maxval, dtype=tf.int32, seed=None, name=None)
+        shape=size, minval=minval, maxval=maxval, dtype=tf.int32, seed=None, name=None
+    )
 
 
 def rand(*args):
@@ -42,3 +44,12 @@ def uniform(low=0.0, high=1.0, size=None):
     if size is None:
         size = (1,)
     return tf.random.uniform(shape=size, minval=low, maxval=high)
+
+
+def multivariate_normal(mean, cov, size=None):
+    if size is None:
+        size = ()
+    return tfd.Sample(
+        tfd.MultivariateNormalFullCovariance(loc=mean, covariance_matrix=cov),
+        sample_shape=size,
+    ).sample()
