@@ -2,6 +2,8 @@
 
 The n-dimensional hyperbolic space embedded with
 the hyperboloid representation (embedded in minkowsky space).
+
+Lead author: Nina Miolane.
 """
 
 import math
@@ -10,11 +12,11 @@ import geomstats.algebra_utils as utils
 import geomstats.backend as gs
 import geomstats.vectorization
 from geomstats.geometry._hyperbolic import HyperbolicMetric, _Hyperbolic
-from geomstats.geometry.base import EmbeddedManifold
+from geomstats.geometry.base import LevelSet
 from geomstats.geometry.minkowski import Minkowski, MinkowskiMetric
 
 
-class Hyperboloid(_Hyperbolic, EmbeddedManifold):
+class Hyperboloid(_Hyperbolic, LevelSet):
     """Class for the n-dimensional hyperboloid space.
 
     Class for the n-dimensional hyperboloid space as embedded in (
@@ -380,10 +382,10 @@ class HyperboloidMetric(HyperbolicMetric):
         angle = self.dist(base_point, point) / self.scale
 
         coef_1_ = utils.taylor_exp_even_func(
-            angle ** 2, utils.inv_sinch_close_0, order=4
+            angle**2, utils.inv_sinch_close_0, order=4
         )
         coef_2_ = utils.taylor_exp_even_func(
-            angle ** 2, utils.inv_tanh_close_0, order=4
+            angle**2, utils.inv_tanh_close_0, order=4
         )
 
         log_term_1 = gs.einsum("...,...j->...j", coef_1_, point)
@@ -417,32 +419,49 @@ class HyperboloidMetric(HyperbolicMetric):
         dist *= self.scale
         return dist
 
-    def parallel_transport(self, tangent_vec_a, tangent_vec_b, base_point):
-        """Compute the parallel transport of a tangent vector.
+    def parallel_transport(
+        self, tangent_vec, base_point, direction=None, end_point=None
+    ):
+        r"""Compute the parallel transport of a tangent vector.
 
-        Closed-form solution for the parallel transport of a tangent vector a
-        along the geodesic defined by exp_(base_point)(tangent_vec_b).
+        Closed-form solution for the parallel transport of a tangent vector
+        along the geodesic between two points `base_point` and `end_point`
+        or alternatively defined by :math:`t\mapsto exp_(base_point)(
+        t*direction)`.
 
         Parameters
         ----------
-        tangent_vec_a : array-like, shape=[..., dim + 1]
+        tangent_vec : array-like, shape=[..., dim + 1]
             Tangent vector at base point to be transported.
-        tangent_vec_b : array-like, shape=[..., dim + 1]
+        base_point : array-like, shape=[..., dim + 1]
+            Point on the hyperboloid.
+        direction : array-like, shape=[..., dim + 1]
             Tangent vector at base point, along which the parallel transport
             is computed.
-        base_point : array-like, shape=[..., dim + 1]
-            Point on the hypersphere.
+            Optional, default : None.
+        end_point : array-like, shape=[..., dim + 1]
+            Point on the hyperboloid. Point to transport to. Unused if `tangent_vec_b`
+            is given.
+            Optional, default : None.
 
         Returns
         -------
         transported_tangent_vec: array-like, shape=[..., dim + 1]
             Transported tangent vector at exp_(base_point)(tangent_vec_b).
         """
-        theta = self.embedding_metric.norm(tangent_vec_b)
+        if direction is None:
+            if end_point is not None:
+                direction = self.log(end_point, base_point)
+            else:
+                raise ValueError(
+                    "Either an end_point or a tangent_vec_b must be given to define the"
+                    " geodesic along which to transport."
+                )
+        theta = self.embedding_metric.norm(direction)
         eps = gs.where(theta == 0.0, 1.0, theta)
-        normalized_b = gs.einsum("...,...i->...i", 1 / eps, tangent_vec_b)
-        pb = self.embedding_metric.inner_product(tangent_vec_a, normalized_b)
-        p_orth = tangent_vec_a - gs.einsum("...,...i->...i", pb, normalized_b)
+        normalized_b = gs.einsum("...,...i->...i", 1 / eps, direction)
+        pb = self.embedding_metric.inner_product(tangent_vec, normalized_b)
+        p_orth = tangent_vec - gs.einsum("...,...i->...i", pb, normalized_b)
         transported = (
             gs.einsum("...,...i->...i", gs.sinh(theta) * pb, base_point)
             + gs.einsum("...,...i->...i", gs.cosh(theta) * pb, normalized_b)
