@@ -1,10 +1,12 @@
 """The n-dimensional hyperbolic space.
 
 The n-dimensional hyperbolic space with Poincare ball model.
+
+Lead author: Hadi Zaatiti.
 """
+
 import geomstats.algebra_utils as utils
 import geomstats.backend as gs
-import geomstats.vectorization
 from geomstats.geometry._hyperbolic import _Hyperbolic
 from geomstats.geometry.base import OpenSet
 from geomstats.geometry.euclidean import Euclidean
@@ -66,7 +68,7 @@ class PoincareBall(_Hyperbolic, OpenSet):
             Array of booleans indicating whether the corresponding points
             belong to the hyperbolic space.
         """
-        return gs.sum(point ** 2, axis=-1) < (1 - atol)
+        return gs.sum(point**2, axis=-1) < (1 - atol)
 
     def projection(self, point):
         """Project a point on the ball.
@@ -136,7 +138,7 @@ class PoincareBallMetric(RiemannianMetric):
             Point in the Poincare ball equal to the Riemannian exponential
             of tangent_vec at the base point.
         """
-        squared_norm_bp = gs.sum(base_point ** 2, axis=-1)
+        squared_norm_bp = gs.sum(base_point**2, axis=-1)
         norm_tan = gs.linalg.norm(tangent_vec, axis=-1)
         lambda_base_point = 1 / (1 - squared_norm_bp)
 
@@ -169,8 +171,8 @@ class PoincareBallMetric(RiemannianMetric):
             of point at the base point.
         """
         mobius_addition = self.mobius_add(-base_point, point)
-        squared_norm_add = gs.sum(mobius_addition ** 2, axis=-1)
-        squared_norm_bp = gs.sum(base_point ** 2, axis=-1)
+        squared_norm_add = gs.sum(mobius_addition**2, axis=-1)
+        squared_norm_bp = gs.sum(base_point**2, axis=-1)
         coef = (1 - squared_norm_bp) * utils.taylor_exp_even_func(
             squared_norm_add, utils.arctanh_card_close_0
         )
@@ -214,8 +216,8 @@ class PoincareBallMetric(RiemannianMetric):
                 raise ValueError("Points do not belong to the Poincare ball")
 
         inner = gs.sum(point_a * point_b, axis=-1)
-        squared_norm_a = gs.sum(point_a ** 2, axis=-1)
-        squared_norm_b = gs.sum(point_b ** 2, axis=-1)
+        squared_norm_a = gs.sum(point_a**2, axis=-1)
+        squared_norm_b = gs.sum(point_b**2, axis=-1)
         num_1 = gs.einsum("...,...i->...i", 1 + 2 * inner + squared_norm_b, point_a)
         num_2 = gs.einsum("...,...i->...i", 1 - squared_norm_a, point_b)
         result = gs.einsum(
@@ -225,7 +227,6 @@ class PoincareBallMetric(RiemannianMetric):
         )
         return ball_manifold.projection(result)
 
-    @geomstats.vectorization.decorator(["else", "vector", "vector"])
     def dist(self, point_a, point_b):
         """Compute the geodesic distance between two points.
 
@@ -241,17 +242,16 @@ class PoincareBallMetric(RiemannianMetric):
         dist : array-like, shape=[...,]
             Geodesic distance between the two points.
         """
-        point_a_norm = gs.clip(gs.sum(point_a ** 2, -1), 0.0, 1 - EPSILON)
-        point_b_norm = gs.clip(gs.sum(point_b ** 2, -1), 0.0, 1 - EPSILON)
+        point_a_norm = gs.clip(gs.sum(point_a**2, -1), 0.0, 1 - EPSILON)
+        point_b_norm = gs.clip(gs.sum(point_b**2, -1), 0.0, 1 - EPSILON)
 
         diff_norm = gs.sum((point_a - point_b) ** 2, -1)
         norm_function = 1 + 2 * diff_norm / ((1 - point_a_norm) * (1 - point_b_norm))
 
-        dist = gs.log(norm_function + gs.sqrt(norm_function ** 2 - 1))
+        dist = gs.log(norm_function + gs.sqrt(norm_function**2 - 1))
         dist *= self.scale
         return dist
 
-    @geomstats.vectorization.decorator(["else", "vector", "vector"])
     def retraction(self, tangent_vec, base_point):
         """Poincaré ball model retraction.
 
@@ -279,12 +279,11 @@ class PoincareBallMetric(RiemannianMetric):
             raise NameError("Points do not belong to the Poincare ball")
 
         retraction_factor = (
-            (1 - gs.sum(base_point ** 2, axis=-1, keepdims=True)) ** 2
+            (1 - gs.sum(base_point**2, axis=-1, keepdims=True)) ** 2
         ) / 4
 
         return base_point - gs.einsum("...i,...j->...j", retraction_factor, tangent_vec)
 
-    @geomstats.vectorization.decorator(["else", "vector"])
     def metric_matrix(self, base_point=None):
         """Compute the inner product matrix.
 
@@ -305,7 +304,7 @@ class PoincareBallMetric(RiemannianMetric):
         lambda_base = (2 / (1 - gs.sum(base_point * base_point, axis=-1))) ** 2
         identity = gs.eye(self.dim, self.dim)
 
-        return gs.einsum("i,jk->ijk", lambda_base, identity)
+        return gs.einsum("...,jk->...jk", lambda_base, identity)
 
     def normalization_factor(self, variances):
         """Return normalization factor of the Gaussian distribution.
@@ -424,7 +423,7 @@ class PoincareBallMetric(RiemannianMetric):
         grad_term_211 = (
             gs.exp((prod_alpha_sigma) ** 2)
             * (1 + gs.erf(prod_alpha_sigma))
-            * gs.einsum("ij,j->ij", sigma_repeated, alpha ** 2)
+            * gs.einsum("ij,j->ij", sigma_repeated, alpha**2)
             * 2
         )
 
