@@ -1,13 +1,14 @@
 """Product of Riemannian metrics.
 
 Define the metric of a product manifold endowed with a product metric.
+
+Lead author: Nicolas Guigui.
 """
 
 import joblib
 
 import geomstats.backend as gs
 import geomstats.errors
-import geomstats.vectorization
 from geomstats.geometry.riemannian_metric import RiemannianMetric
 
 
