@@ -1,4 +1,7 @@
-"""The manifold of symmetric positive definite (SPD) matrices."""
+"""The manifold of symmetric positive definite (SPD) matrices.
+
+Lead author: Yann Thanwerdas.
+"""
 
 import math
 
@@ -7,6 +10,9 @@ import geomstats.vectorization
 from geomstats.geometry.base import OpenSet
 from geomstats.geometry.general_linear import GeneralLinear
 from geomstats.geometry.matrices import Matrices
+from geomstats.geometry.positive_lower_triangular_matrices import (
+    PositiveLowerTriangularMatrices,
+)
 from geomstats.geometry.riemannian_metric import RiemannianMetric
 from geomstats.geometry.symmetric_matrices import SymmetricMatrices
 
@@ -45,10 +51,9 @@ class SPDMatrices(OpenSet):
         belongs : array-like, shape=[...,]
             Boolean denoting if mat is an SPD matrix.
         """
-        is_symmetric = self.ambient_space.belongs(mat, atol)
-        eigvalues = gs.linalg.eigvalsh(mat)
-        is_positive = gs.all(eigvalues > 0, axis=-1)
-        belongs = gs.logical_and(is_symmetric, is_positive)
+        is_sym = self.ambient_space.belongs(mat, atol)
+        is_pd = Matrices.is_pd(mat)
+        belongs = gs.logical_and(is_sym, is_pd)
         return belongs
 
     def projection(self, point):
@@ -123,8 +128,10 @@ class SPDMatrices(OpenSet):
 
         sqrt_base_point = gs.linalg.sqrtm(base_point)
 
-        tangent_vec_at_id = 2 * gs.random.rand(*size) - 1
-        tangent_vec_at_id += Matrices.transpose(tangent_vec_at_id)
+        tangent_vec_at_id_aux = 2 * gs.random.rand(*size) - 1
+        tangent_vec_at_id = tangent_vec_at_id_aux + Matrices.transpose(
+            tangent_vec_at_id_aux
+        )
 
         tangent_vec = Matrices.mul(sqrt_base_point, tangent_vec_at_id, sqrt_base_point)
 
@@ -161,7 +168,7 @@ class SPDMatrices(OpenSet):
         elif power == math.inf:
             powered_eigvalues = gs.exp(eigvalues)
         else:
-            powered_eigvalues = eigvalues ** power
+            powered_eigvalues = eigvalues**power
 
         denominator = eigvalues[..., :, None] - eigvalues[..., None, :]
         numerator = powered_eigvalues[..., :, None] - powered_eigvalues[..., None, :]
@@ -412,6 +419,51 @@ class SPDMatrices(OpenSet):
     from_vector = SymmetricMatrices.__dict__["from_vector"]
     to_vector = SymmetricMatrices.__dict__["to_vector"]
 
+    @classmethod
+    def cholesky_factor(cls, mat):
+        """Compute cholesky factor.
+
+        Compute cholesky factor for a symmetric positive
+        definite matrix.
+
+        Parameters
+        ----------
+        mat : array_like, shape=[..., n, n]
+            spd matrix.
+
+        Returns
+        -------
+        cf : array_like, shape=[..., n, n]
+            lower triangular matrix with positive diagonal elements.
+        """
+        return gs.linalg.cholesky(mat)
+
+    @classmethod
+    def differential_cholesky_factor(cls, tangent_vec, base_point):
+        """Compute the differential of the cholesky factor map.
+
+        Parameters
+        ----------
+        tangent_vec : array_like, shape=[..., n, n]
+            Tangent vector at base point.
+            symmetric matrix.
+
+        base_point : array_like, shape=[..., n, n]
+            Base point.
+            spd matrix.
+
+        Returns
+        -------
+        differential_cf : array-like, shape=[..., n, n]
+            Differential of cholesky factor map
+            lower triangular matrix.
+        """
+        cf = cls.cholesky_factor(base_point)
+        differential_cf = PositiveLowerTriangularMatrices.inverse_differential_gram(
+            tangent_vec, cf
+        )
+        return differential_cf
+
 
 class SPDMetricAffine(RiemannianMetric):
     """Class for the affine-invariant metric on the SPD manifold."""
@@ -502,7 +554,7 @@ class SPDMetricAffine(RiemannianMetric):
                 modified_tangent_vec_a, modified_tangent_vec_b, power_inv_base_point
             )
 
-            inner_product = inner_product / (power_affine ** 2)
+            inner_product = inner_product / (power_affine**2)
 
         return inner_product
 
@@ -623,39 +675,47 @@ class SPDMetricAffine(RiemannianMetric):
             log = SPDMatrices.inverse_differential_power(power_affine, log, base_point)
         return log
 
-    def parallel_transport(self, tangent_vec_a, tangent_vec_b, base_point):
+    def parallel_transport(
+        self, tangent_vec, base_point, direction=None, end_point=None
+    ):
         r"""Parallel transport of a tangent vector.
 
-        Closed-form solution for the parallel transport of a tangent vector a
-        along the geodesic defined by exp_(base_point)(tangent_vec_b).
-        Denoting `tangent_vec_a` by `S`, `base_point` by `A`, let
-        `B = Exp_A(tangent_vec_b)` and :math: `E = (BA^{- 1})^({ 1 / 2})`.
-        Then the
-        parallel transport to `B`is:
+        Closed-form solution for the parallel transport of a tangent vector
+        along the geodesic between two points `base_point` and `end_point`
+        or alternatively defined by :math:`t\mapsto exp_(base_point)(
+        t*direction)`.
+        Denoting `tangent_vec_a` by `S`, `base_point` by `A`, and `end_point`
+        by `B` or `B = Exp_A(tangent_vec_b)` and :math: `E = (BA^{- 1})^({ 1
+        / 2})`. Then the parallel transport to `B` is:
 
         ..math::
                         S' = ESE^T
 
         Parameters
         ----------
-        tangent_vec_a : array-like, shape=[..., dim + 1]
+        tangent_vec : array-like, shape=[..., n, n]
             Tangent vector at base point to be transported.
-        tangent_vec_b : array-like, shape=[..., dim + 1]
+        base_point : array-like, shape=[..., n, n]
+            Point on the manifold of SPD matrices. Point to transport from
+        direction : array-like, shape=[..., n, n]
             Tangent vector at base point, initial speed of the geodesic along
-            which the parallel transport is computed.
-        base_point : array-like, shape=[..., dim + 1]
-            Point on the manifold of SPD matrices.
+            which the parallel transport is computed. Unused if `end_point` is given.
+            Optional, default: None.
+        end_point : array-like, shape=[..., n, n]
+            Point on the manifold of SPD matrices. Point to transport to.
+            Optional, default: None.
 
         Returns
         -------
-        transported_tangent_vec: array-like, shape=[..., dim + 1]
+        transported_tangent_vec: array-like, shape=[..., n, n]
             Transported tangent vector at exp_(base_point)(tangent_vec_b).
         """
-        end_point = self.exp(tangent_vec_b, base_point)
+        if end_point is None:
+            end_point = self.exp(direction, base_point)
         inverse_base_point = GeneralLinear.inverse(base_point)
         congruence_mat = Matrices.mul(end_point, inverse_base_point)
         congruence_mat = gs.linalg.sqrtm(congruence_mat)
-        return Matrices.congruent(tangent_vec_a, congruence_mat)
+        return Matrices.congruent(tangent_vec, congruence_mat)
 
 
 class SPDMetricBuresWasserstein(RiemannianMetric):
@@ -845,7 +905,7 @@ class SPDMetricEuclidean(RiemannianMetric):
 
             inner_product = Matrices.frobenius_product(
                 modified_tangent_vec_a, modified_tangent_vec_b
-            ) / (power_euclidean ** 2)
+            ) / (power_euclidean**2)
         return inner_product
 
     @staticmethod
@@ -883,6 +943,104 @@ class SPDMetricEuclidean(RiemannianMetric):
 
         return domain
 
+    def exp(self, tangent_vec, base_point, **kwargs):
+        """Compute the Euclidean exponential map.
+
+        Compute the Euclidean exponential at point base_point
+        of tangent vector tangent_vec.
+        This gives a symmetric positive definite matrix.
+
+        Parameters
+        ----------
+        tangent_vec : array-like, shape=[..., n, n]
+            Tangent vector at base point.
+        base_point : array-like, shape=[..., n, n]
+            Base point.
+
+        Returns
+        -------
+        exp : array-like, shape=[..., n, n]
+            Euclidean exponential.
+        """
+        power_euclidean = self.power_euclidean
+
+        if power_euclidean == 1:
+            exp = tangent_vec + base_point
+        else:
+            exp = SymmetricMatrices.powerm(
+                SymmetricMatrices.powerm(base_point, power_euclidean)
+                + SPDMatrices.differential_power(
+                    power_euclidean, tangent_vec, base_point
+                ),
+                1 / power_euclidean,
+            )
+        return exp
+
+    def log(self, point, base_point, **kwargs):
+        """Compute the Euclidean logarithm map.
+
+        Compute the Euclidean logarithm at point base_point, of point.
+        This gives a tangent vector at point base_point.
+
+        Parameters
+        ----------
+        point : array-like, shape=[..., n, n]
+            Point.
+        base_point : array-like, shape=[..., n, n]
+            Base point.
+
+        Returns
+        -------
+        log : array-like, shape=[..., n, n]
+            Euclidean logarithm.
+        """
+        power_euclidean = self.power_euclidean
+
+        if power_euclidean == 1:
+            log = point - base_point
+        else:
+            log = SPDMatrices.inverse_differential_power(
+                power_euclidean,
+                SymmetricMatrices.powerm(point, power_euclidean)
+                - SymmetricMatrices.powerm(base_point, power_euclidean),
+                base_point,
+            )
+
+        return log
+
+    def parallel_transport(
+        self, tangent_vec, base_point, direction=None, end_point=None
+    ):
+        r"""Compute the parallel transport of a tangent vector.
+
+        Closed-form solution for the parallel transport of a tangent vector
+        along the geodesic between two points `base_point` and `end_point`
+        or alternatively defined by :math:`t\mapsto exp_(base_point)(
+        t*direction)`.
+
+        Parameters
+        ----------
+        tangent_vec : array-like, shape=[..., n, n]
+            Tangent vector at base point to be transported.
+        base_point : array-like, shape=[..., n, n]
+            Point on the manifold. Point to transport from.
+        direction : array-like, shape=[..., n, n]
+            Tangent vector at base point, along which the parallel transport
+            is computed.
+            Optional, default: None.
+        end_point : array-like, shape=[..., n, n]
+            Point on the manifold. Point to transport to.
+            Optional, default: None.
+
+        Returns
+        -------
+        transported_tangent_vec: array-like, shape=[..., n, n]
+            Transported tangent vector at `exp_(base_point)(tangent_vec_b)`.
+        """
+        if self.power_euclidean == 1:
+            return tangent_vec
+        raise NotImplementedError("Parallel transport is only implemented for power 1")
+
 
 class SPDMetricLogEuclidean(RiemannianMetric):
     """Class for the Log-Euclidean metric on the SPD manifold.
