@@ -1,4 +1,7 @@
-"""Visualization for Geometric Statistics."""
+"""Visualization for Geometric Statistics.
+
+Lead authors: Nina Miolane and Elodie Maignant.
+"""
 import logging
 
 import matplotlib
@@ -45,10 +48,12 @@ IMPLEMENTED = [
     "M32",
     "S33",
     "M33",
+    "SPD2",
 ]
 
 
 def tutorial_matplotlib():
+    """Set default parameters for matplotlib."""
     fontsize = 12
     matplotlib.rc("font", size=fontsize)
     matplotlib.rc("text")
@@ -230,8 +235,11 @@ class Sphere:
                     )
 
     def fibonnaci_points(self, n_points=16000):
-        """Spherical Fibonacci point sets yield nearly uniform point
-        distributions on the unit sphere."""
+        """Plot spherical Fibonacci point set.
+
+        Spherical Fibonacci point set yields nearly uniform point
+        distributions on the unit sphere.
+        """
         x_vals = []
         y_vals = []
         z_vals = []
@@ -409,9 +417,9 @@ class PoincareHalfPlane:
         disk_x = disk_coords[:, 0]
         disk_y = disk_coords[:, 1]
 
-        denominator = disk_x ** 2 + (1 - disk_y) ** 2
+        denominator = disk_x**2 + (1 - disk_y) ** 2
         coords_0 = gs.expand_dims(2 * disk_x / denominator, axis=1)
-        coords_1 = gs.expand_dims((1 - disk_x ** 2 - disk_y ** 2) / denominator, axis=1)
+        coords_1 = gs.expand_dims((1 - disk_x**2 - disk_y**2) / denominator, axis=1)
 
         half_plane_coords = gs.concatenate([coords_0, coords_1], axis=1)
         return half_plane_coords
@@ -451,7 +459,7 @@ class KleinDisk:
         poincare_radius = gs.linalg.norm(poincare_coords, axis=1)
         poincare_angle = gs.arctan2(poincare_coords[:, 1], poincare_coords[:, 0])
 
-        klein_radius = 2 * poincare_radius / (1 + poincare_radius ** 2)
+        klein_radius = 2 * poincare_radius / (1 + poincare_radius**2)
         klein_angle = poincare_angle
 
         coords_0 = gs.expand_dims(klein_radius * gs.cos(klein_angle), axis=1)
@@ -520,6 +528,99 @@ class SpecialEuclidean2:
         ax.scatter(translation[:, 0], translation[:, 1], s=16, **kwargs)
 
 
+class Ellipses:
+    """Class used to plot points on the manifold SPD(2).
+
+    Elements S of the manifold of 2D Symmetric Positive Definite matrices
+    can be conveniently represented by ellipses.
+
+    We write :math: `S = O D O^T` with :math: `O` an orthogonal matrix (rotation)
+    and :math: `D` a diagonal matrix. The positive eigenvalues, i.e. the elements of
+    :math: `D`, are the inverse of the length of the major and minor axes of
+    the ellipse. The rotation matrix :math: `O` determines the orientation of the
+    2D ellipse in the 2D plane.
+
+    Parameters
+    ----------
+    n_sampling_points : int
+        Number of points to sample on the discretized ellipses.
+    """
+
+    def __init__(self, n_sampling_points=100):
+        self.n_sampling_points = n_sampling_points
+
+    @staticmethod
+    def set_ax(ax=None):
+        """Set the axis for the Figure.
+
+        Parameters
+        ----------
+        ax : Axis
+            Axis of the figure.
+
+        Returns
+        -------
+        ax : Axis
+            Axis of the figure.
+        """
+        if ax is None:
+            ax = plt.subplot()
+        plt.setp(ax, xlabel="X", ylabel="Y")
+        return ax
+
+    def draw_points(self, points=None, ax=None, **plot_kwargs):
+        """Draw the ellipses.
+
+        Parameters
+        ----------
+        ax : Axis
+            Axis of the figure.
+        points : array-like, shape=[..., 2, 2]
+            Points on the SPD manifold of 2D symmetric
+            positive definite matrices.
+            Optional, default: None.
+        plot_kwargs : dict
+            Dictionnary of arguments related to plotting.
+        """
+        if ax is None:
+            ax = self.set_ax()
+        if points.ndim == 2:
+            points = [points]
+        for point in points:
+            x_coords, y_coords = self.compute_coordinates(point)
+            ax.plot(x_coords, y_coords, **plot_kwargs)
+
+    def compute_coordinates(self, point):
+        """Compute the ellipse coordinates of a 2D SPD matrix.
+
+        Parameters
+        ----------
+        point : array-like, shape=[2, 2]
+            SPD matrix.
+
+        Returns
+        -------
+        x_coords : array-like, shape=[n_sampling_points,]
+            x_coords coordinates of the sampling points on the discretized ellipse.
+        Y: array-like, shape = [n_sampling_points,]
+            y coordinates of the sampling points on the discretized ellipse.
+        """
+        eigvalues, eigvectors = gs.linalg.eigh(point)
+        eigvalues = gs.where(eigvalues < gs.atol, gs.atol, eigvalues)
+
+        [eigvalue1, eigvalue2] = eigvalues
+
+        rot_sin = eigvectors[1, 0]
+        rot_cos = eigvectors[0, 0]
+        thetas = gs.linspace(0.0, 2 * gs.pi, self.n_sampling_points + 1)
+
+        x_coords = eigvalue1 * gs.cos(thetas) * rot_cos
+        x_coords -= rot_sin * eigvalue2 * gs.sin(thetas)
+        y_coords = eigvalue1 * gs.cos(thetas) * rot_sin
+        y_coords += rot_cos * eigvalue2 * gs.sin(thetas)
+        return x_coords, y_coords
+
+
 class KendallSphere:
     """Class used to plot points in Kendall shape space of 2D triangles.
 
@@ -815,16 +916,13 @@ class KendallDisk:
             [[1.0, 0.0], [-0.5, gs.sqrt(3.0) / 2.0], [-0.5, -gs.sqrt(3.0) / 2.0]]
         ) / gs.sqrt(3.0)
 
-        self.centre = (
-            gs.array(
-                [
-                    [1.0, 0.0, 0.0],
-                    [-0.5, gs.sqrt(3.0) / 2.0, 0.0],
-                    [-0.5, -gs.sqrt(3.0) / 2.0, 0.0],
-                ]
-            )
-            / gs.sqrt(3.0)
-        )
+        self.centre = gs.array(
+            [
+                [1.0, 0.0, 0.0],
+                [-0.5, gs.sqrt(3.0) / 2.0, 0.0],
+                [-0.5, -gs.sqrt(3.0) / 2.0, 0.0],
+            ]
+        ) / gs.sqrt(3.0)
 
         self.ua = gs.array(
             [[-1.0, 0.0], [0.5, gs.sqrt(3.0) / 2.0], [0.5, -gs.sqrt(3.0) / 2.0]]
@@ -1007,8 +1105,7 @@ class KendallDisk:
 def convert_to_trihedron(point, space=None):
     """Transform a rigid point into a trihedron.
 
-    Transform a rigid point
-    into a trihedron s.t.:
+    Transform a rigid point into a trihedron such that:
     - the trihedron's base point is the translation of the origin
     of R^3 by the translation part of point,
     - the trihedron's orientation is the rotation of the canonical basis
@@ -1067,7 +1164,7 @@ def plot(points, ax=None, space=None, point_type=None, **point_draw_kwargs):
         Points to be plotted.
     space: str, optional, {'SO3_GROUP', 'SE3_GROUP', 'S1', 'S2',
         'H2_poincare_disk', 'H2_poincare_half_plane', 'H2_klein_disk',
-        'poincare_polydisk', 'S32', 'M32', 'S33', 'M33'}
+        'poincare_polydisk', 'S32', 'M32', 'S33', 'M33', 'SPD2'}
     point_type: str, optional, {'extrinsic', 'ball', 'half-space', 'pre-shape'}
     """
     if space not in IMPLEMENTED:
@@ -1131,7 +1228,7 @@ def plot(points, ax=None, space=None, point_type=None, **point_draw_kwargs):
             point_type = "extrinsic"
         n_disks = points.shape[1]
         poincare_poly_disk = PoincarePolyDisk(point_type=point_type, n_disks=n_disks)
-        n_columns = int(gs.ceil(n_disks ** 0.5))
+        n_columns = int(gs.ceil(n_disks**0.5))
         n_rows = int(gs.ceil(n_disks / n_columns))
 
         axis_list = []
@@ -1193,4 +1290,8 @@ def plot(points, ax=None, space=None, point_type=None, **point_draw_kwargs):
         disk.draw_points()
         ax = disk.ax
 
+    elif space == "SPD2":
+        ellipses = Ellipses()
+        ellipses.draw_points(points=points)
+
     return ax
