@@ -1,5 +1,7 @@
 """Unit tests for errors."""
 
+import pytest
+
 import geomstats.backend as gs
 import geomstats.errors
 import geomstats.tests
@@ -12,9 +14,8 @@ class TestBackends(geomstats.tests.TestCase):
         euclidean = Euclidean(5)
         point = gs.array([1, 2])
 
-        self.assertRaises(
-            RuntimeError, lambda: geomstats.errors.check_belongs(point, euclidean)
-        )
+        with pytest.raises(RuntimeError):
+            geomstats.errors.check_belongs(point, euclidean)
 
     @staticmethod
     def test_check_belongs_with_tol():
@@ -26,14 +27,13 @@ class TestBackends(geomstats.tests.TestCase):
     def test_check_integer(self):
         a = -2
 
-        self.assertRaises(ValueError, lambda: geomstats.errors.check_integer(a, "a"))
+        with pytest.raises(ValueError):
+            geomstats.errors.check_integer(a, "a")
 
     def test_check_parameter_accepted_values(self):
         param = "lefttt"
         accepted_values = ["left", "right"]
-        self.assertRaises(
-            ValueError,
-            lambda: geomstats.errors.check_parameter_accepted_values(
+        with pytest.raises(ValueError):
+            geomstats.errors.check_parameter_accepted_values(
                 param, "left_or_right", accepted_values
-            ),
-        )
+            )
