@@ -2,6 +2,8 @@
 
 import math
 
+import pytest
+
 import geomstats.backend as gs
 import geomstats.tests
 import tests.helper as helper
@@ -17,7 +19,7 @@ RTOL = 1e-6
 
 
 class TestHyperbolic(geomstats.tests.TestCase):
-    def setUp(self):
+    def setup_method(self):
         gs.random.seed(1234)
         self.dimension = 3
         self.space = Hyperboloid(dim=self.dimension)
@@ -41,10 +43,11 @@ class TestHyperbolic(geomstats.tests.TestCase):
 
     def test_regularize_zero_norm(self):
         point = gs.array([-1.0, 1.0, 0.0, 0.0])
-        self.assertRaises(ValueError, lambda: self.space.regularize(point))
-        self.assertRaises(
-            NameError, lambda: self.space.extrinsic_to_intrinsic_coords(point)
-        )
+        with pytest.raises(ValueError):
+            self.space.regularize(point)
+
+        with pytest.raises(NameError):
+            self.space.extrinsic_to_intrinsic_coords(point)
 
     def test_random_uniform_and_belongs(self):
         point = self.space.random_point()
@@ -494,7 +497,7 @@ class TestHyperbolic(geomstats.tests.TestCase):
             tangent_vec_a, tangent_vec_b, base_point
         )
         result = inner_product_scaled_metric
-        expected = scale ** 2 * inner_product_default_metric
+        expected = scale**2 * inner_product_default_metric
         self.assertAllClose(result, expected)
 
     def test_scaled_squared_norm(self):
@@ -512,7 +515,7 @@ class TestHyperbolic(geomstats.tests.TestCase):
             tangent_vec, base_point
         )
         result = squared_norm_scaled_metric
-        expected = scale ** 2 * squared_norm_default_metric
+        expected = scale**2 * squared_norm_default_metric
         self.assertAllClose(result, expected)
 
     def test_scaled_distance(self):
