@@ -3,6 +3,8 @@
 import logging
 import warnings
 
+import pytest
+
 import geomstats.backend as gs
 import geomstats.tests
 import tests.helper as helper
@@ -14,7 +16,7 @@ from geomstats.geometry.special_orthogonal import SpecialOrthogonal
 
 
 class TestInvariantMetric(geomstats.tests.TestCase):
-    def setUp(self):
+    def setup_method(self):
         logger = logging.getLogger()
         logger.disabled = True
         warnings.simplefilter("ignore", category=ImportWarning)
@@ -530,7 +532,7 @@ class TestInvariantMetric(geomstats.tests.TestCase):
         basis = metric.normal_basis(group.lie_algebra.basis)
         x, y, z = basis
         result = metric.structure_constant(x, y, z)
-        expected = 2.0 ** 0.5 / 2.0
+        expected = 2.0**0.5 / 2.0
         self.assertAllClose(result, expected)
 
         result = -metric.structure_constant(y, x, z)
@@ -570,7 +572,7 @@ class TestInvariantMetric(geomstats.tests.TestCase):
         metric = InvariantMetric(group=group)
         x, y, z = metric.normal_basis(group.lie_algebra.basis)
         result = metric.connection(x, y)
-        expected = 1.0 / 2 ** 0.5 / 2.0 * z
+        expected = 1.0 / 2**0.5 / 2.0 * z
         self.assertAllClose(result, expected)
 
         point = group.random_uniform()
@@ -764,11 +766,17 @@ class TestInvariantMetric(geomstats.tests.TestCase):
             "...ij,...->...ij", tan_a, 1.0 / metric.norm(tan_a, base_point=point)
         )
 
-        expected = group.left_canonical_metric.parallel_transport(tan_a, tan_b, point)
+        expected = group.left_canonical_metric.parallel_transport(tan_a, point, tan_b)
         result, end_point_result = metric.parallel_transport(
-            tan_a, tan_b, point, n_steps=20, step="rk4", return_endpoint=True
+            tan_a, point, tan_b, n_steps=20, step="rk4", return_endpoint=True
         )
         expected_end_point = metric.exp(tan_b, point, n_steps=20)
 
         self.assertAllClose(end_point_result, expected_end_point)
         self.assertAllClose(expected, result)
+
+    def test_log_antipodals(self):
+        rotation_mat1 = gs.eye(3)
+        rotation_mat2 = gs.array([[1.0, 0.0, 0.0], [0.0, -1.0, 0.0], [0.0, 0.0, -1.0]])
+        with pytest.raises(ValueError):
+            self.matrix_so3.bi_invariant_metric.log(rotation_mat1, rotation_mat2)
